/*
 WLib Javascript Library
 Auhtor: Matthias.Wiede@web.de
 
 WDragable
 
*/

function WDragable (name, target, attr)
{
	/* Init vars */
	this.target = target;
	this.name = name;
	this.attr = attr;
	
	/* Temporary and status vars */
	this.wnd = null;
	this.content = null;	
	this.header = null;
	this.fResize = false;
	this.moveXDiff = 0;
	this.moveYDiff = 0;
	this.convertedPos = {x:0, y:0};
	this.oldMousePos = {x:0, y:0};

	this.onMouseDown = function (evt) 
	{
		var div= (evt.target) ? evt.target : evt.srcElement;
		var dragable = WUI.dragableList[div.id];
		if (dragable && evt.button!=2) {
			dragable.captureMouse (evt);
			
			$('#'+dragable.target.id+' .dragable').removeClass ('selected');
			if (dragable.attr.contextmenuFunc) {
				dragable.attr.contextmenuFunc (null, null, dragable.wnd.id, { x: dragable.convertedPos.x, y : dragable.convertedPos.y, cmd : 'capture' });

				$(dragable.wnd).addClass ('selected');
			}

		}		
		return false;
	}
	
	this.onMouseDblClick = function (evt) 
	{
		var div= (evt.target) ? evt.target : evt.srcElement;
		var dragable = WUI.dragableList[div.id];
		if (dragable && evt.button!=2) {
		
			if (dragable.attr.contextmenuFunc) {
				evt.stopPropagation();
				evt.preventDefault ();
				dragable.attr.contextmenuFunc (null, null, dragable.wnd.id, { x: dragable.convertedPos.x, y : dragable.convertedPos.y, cmd : 'dblclick' });


			}

		}		
		return false;
	}	
	
	this.onMouseUp = function (evt) 
	{
		if (evt.button==2)
			return false;
	
		var dragable = WUI.mouseCaptureParam;
		if (dragable) {
			dragable.releaseMouse ();
			if (dragable.attr.contextmenuFunc) {
				dragable.attr.contextmenuFunc (null, null, dragable.wnd.id, { x: dragable.convertedPos.x, y : dragable.convertedPos.y, cmd : 'release' });
			}
		}		
		return false;
	}

		
	this.onMouseMove = function (evt, dragable) 
	{
		var div= (evt.target) ? evt.target : evt.srcElement;
		if (!dragable)
			dragable = WUI.dragableList[div.id];
		var x  = WUI.mouseCoords.x-dragable.moveXDiff-dragable.target.offsetLeft;
		var y  = WUI.mouseCoords.y-dragable.moveYDiff-dragable.target.offsetTop;
		if (x<0) {
			x=0;
		}
		if (x+dragable.wnd.offsetWidth>dragable.target.offsetWidth)
			x=dragable.target.offsetWidth-dragable.wnd.offsetWidth;
		dragable.wnd.style.left = x+"px";
		if (y<0) {
			y=0
		}
		if (y+dragable.wnd.offsetHeight>dragable.target.offsetHeight)
			y=dragable.target.offsetHeight-dragable.wnd.offsetHeight;
		
		dragable.wnd.style.top  = y+"px";
		dragable.convertedPos = { x: x, y: y};
		
		return false;
	}	
		
	this.remove = function () {
		if (this.attr.close_script)
			WSystem.evalScript (this.attr.close_script)

		this.target.removeChild (this.wnd);
		
		
		delete WUI.dragableList[this.name];

		if (this.attr.addClass)	{
			jQuery ('#'+this.attr.addClass[0]).removeClass (this.attr.addClass[1]);
		}
	}

	this.captureMouse  = function (evt) {

		WUI.captureMouse (this.onMouseMove, this.onMouseUp, this);

		document.body.style.cursor = 'move';
		this.wnd.style.cursor = 'move';

		var pos = WSystem.getAbsoluteCoords (this.wnd);
		this.moveXDiff = WUI.mouseCoords.x - pos.x;
		this.moveYDiff = WUI.mouseCoords.y - pos.y;
		this.oldMousePos = WUI.mouseCoords;
	
		// prevent text selection		
		document.onselectstart = function () { return false; };  
		this.wnd.ondragstart = function() { return false; }; 

		/*	
		evt.stopPropagation();
		evt.preventDefault ();
	
		*/
	}

	this.releaseMouse = function () {		
		WUI.releaseMouse ();		
		// allow text selection
		document.onselectstart = null; 
		this.wnd.ondragstart = null; 
		document.body.style.cursor = 'default';



	}

	this.create = function (name, target, attr)
	{
		// Set default attributes
		var wnd = document.createElement('div');
		wnd.id = name;
		wnd.style.position = "absolute";
		wnd.style.display = "block";

		if (attr.addClass)
			wnd.className = 'dragable '+attr.addClass; 
		else
			wnd.className = 'dragable';
					
		var x = attr.x;
		var y = attr.y;

		wnd.style.left = x+"px";
		wnd.style.top  = y+"px";
		
		if (attr.background)
			wnd.style.backgroundImage = 'url('+attr.background+')';

		if (attr.width)
			wnd.style.width = attr.width+"px";

		if (attr.zIndex) {
			$(wnd).css ('z-index', attr.zIndex);
		}

		if (attr.title)
			$(wnd).attr ('title', attr.title);

		if (attr.height)
			wnd.style.height = attr.height+"px";

		if (attr.text) {
			wnd.innerHTML=attr.text;
		}

		if (attr.contextmenuFunc) {
			jQuery (wnd).bind ('mousedown', attr.contextmenuFunc);
		}
		// Header (movable)

		WSystem.addEvent (wnd, "mousedown", this.onMouseDown);
		WSystem.addEvent (wnd, "mouseup", this.onMouseUp);	
		WSystem.addEvent (wnd, "dblclick", this.onMouseDblClick);	
		wnd.style.cursor='move';

		target.appendChild (wnd);
		this.wnd = wnd;
		this.convertedPos = { x: x, y: y};
		if (this.attr.contextmenuFunc) {
			this.attr.contextmenuFunc (null, null, name, { x: x, y : y, cmd : 'insert' });
		}
		$('#'+target.id+' .dragable').removeClass ('selected');
		$(wnd).addClass ('selected');
	}
	
	// Create dragable	
	this.create (name, target, attr);
	WUI.dragableList[name]=this;
}
