<?php

/**

 WForm
 Form and command handling

**/

abstract class WForm extends WUIElement
{
	protected $cmd = "";
	protected $successText = "";
	protected $errText = "";
	protected $successURI = "";
	protected $actionURI = "";
	protected $fCheckUnsavedModification = true;
	private   $values = NULL;
	
	// Overwritten by child
	public abstract function getDefinition ();
	public abstract function createMask ($cmd, $values);	
	public abstract function getNewData ();
	public abstract function getEditData ();
	public abstract function insert ($values);
	public abstract function update ($values);
	public abstract function search ($values);
	public abstract function delete ($values);
		
	public function setSuccessURI ($uri) {
		$this->successURI = $uri;
	}
	
	public function setActionURI ($uri) {
		$this->actionURI = $uri;
	}
	
	public function getActionURI () {
		return $this->actionURI;
	}
	
	public function checkUnsavedModification ($on) {
		$this->fCheckUnsavedModification = $on;
	}
	
	
	private function updateMessage () {
		if (!empty ($this->successText))
			WUI::setMessage ($this->successText, "success");
		else
		if (!empty ($this->errText))
			WUI::setMessage ($this->errText, "warning");
	}
	
	public function updateInputValues ($values) {
		$fields = $this->getDefinition ();
		foreach ($values as $name => $value) {
			if (isset ($fields[$name])) {
				$list = explode (';', $fields[$name]);

				$this->values[$name]["val"] = $value;
				$this->values[$name]["valf"] = WFormat::format ($value, $list[0]);
			}
		}
	}
	
	public function isUploadFile ($name) {
		return (isset ($_POST[$name."_tmp"]) && is_file (UPLOAD_PATH."/".$_POST[$name."_tmp"]));
	}
	
	public function getUploadFilename ($name, $fOriginal=false) {
		$extra = "";
		if ($fOriginal)
			$extra = "_orig";
		if (isset ($_POST[$name."_tmp"]) && is_file (UPLOAD_PATH."/".$_POST[$name."_tmp"])) {
			return $_POST[$name."_tmp".$extra];
		}
		return "";
	}
	
	public function hideInputForm () {
		$this->updateMessage ();
		WUI::updateForm (array (), $this->id."_hideable");
	}
	
	public function createHideableFrame ($html) {
		return '<div class="hideable" id="'.$this->id.'_hideable">'.$html.'</div>';
	}
	
	public function renderElement () {

		$id = $this->id;
		$cmd = $this->cmd;
		$fields = $this->getDefinition ();
		$this->errText = "";
		$this->successText = "";		
		$this->values = array ();
		
		switch (WSystem::getParam ("cmd")) {
			case $id."_new":		


				if (!WFormUtils::readInput ($fields, $this->values)) {
					
					$ret = $this->insert (WFormUtils::getParsedInputValues ($this->values));
					
					$this->updateMessage ();
					if ($ret) {
						if ($this->fCheckUnsavedModification)
							WUI::checkUnsavedModification (false);	
						if (!empty ($this->successURI))
							WUI::redirect ($this->successURI);
						else {

							WUI::updateForm ($this->values);
						}
					}
					else {
						WUI::updateForm ($this->values);	
					}
				}
				else {
					// Error while parsing

					WUI::setMessage (WLanguage::CHECK_MARKED_FIELDS, "warning");		
					WUI::updateForm ($this->values);
					

				}
				break;
				
			case $id."_edit":	

				if (!WFormUtils::readInput ($fields, $this->values)) {

					$ret = $this->update (WFormUtils::getParsedInputValues ($this->values));
					$this->updateMessage ();

					if ($ret) {
						if ($this->fCheckUnsavedModification)
							WUI::checkUnsavedModification (false);	
						
						if (!empty ($this->successURI))
							WUI::redirect ($this->successURI);
						else {						
							WUI::updateForm ($this->values);
						}
		
					}
					else {

						WUI::updateForm ($this->values);	
					}
				}
				else {
					WUI::setMessage (WLanguage::CHECK_MARKED_FIELDS, "warning");
					WUI::updateForm ($this->values);
				}
				break;
				
			case $id."_search":	
				if (!WFormUtils::readInput ($fields, $this->values)) {
					if (!$this->search (WFormUtils::getParsedInputValues ($this->values))) {					
						$this->updateMessage ();
						WUI::updateForm ($this->values);
					}
				}
				else {
					WUI::setMessage (WLanguage::CHECK_MARKED_FIELDS, "warning");
					WUI::updateForm ($this->values);
				}
				break;
				
			case $id."_delete":	
				WFormUtils::readInput ($fields, $this->values);
				if ($this->delete (WFormUtils::getParsedInputValues ($this->values))) {

					WUI::checkUnsavedModification (false);	
					WUI::setMessage ((empty ($this->successText) ? WLanguage::ENTRY_WAS_DELETED : $this->successText), "success");
					WUI::updateForm (array (), $id."_hideable");

				}
				else {
				
					$this->updateMessage ();
					WUI::updateForm ($this->values);
				}
				break;
				
			default:
				// Initialization
				switch ($cmd) {
					case "new":
						$data = $this->getNewData ();
						break;
					default:
						$data = $this->getEditData ();
						break;
				}

				WFormUtils::initInput ($fields, $this->values, $data);
				break;
		}
		if ($this->fCheckUnsavedModification && ($cmd=="new" || $cmd=="edit")) {
			WUI::checkUnsavedModification (true);	
		}

		$this->html =  
			WUI::createForm ($id,
				WUI::createHiddenField ("cmd", $id."_".$cmd).
				$this->createMask ($cmd, WFormUtils::getFormattedInputValues ($this->values)),
				$this->actionURI
			);

		if (!$this->fRenderOnlyContent) {
			$this->html = '<div id="wrap_'.$this->id.'">'.$this->html.'</div>';
		}

	}
	
	public function createCommandButtons ($cmd, $props=array ()) {
		if ($cmd=="new")
			return WUI::createButton (WSystem::getArrayValue ($props, "create_text", WLanguage::CREATE), "submit", WUI::BUTTON_SUBMIT, array ("extra-class"=>"new_button"));
		else
		if ($cmd=="search") {
			/*		
			$htmlSearch = "<input type=\"button\" class=\"button search_button\" onclick=\"WUI.".WUI::$ajaxMethod." ('".WSystem::getRequestURI (array ("view"=>"search"), true)."', '".WUI::$subPageId."', '', '', false);return false;\" value=\"<!-->".htmlspecialchars (WLanguage::SEARCH)."<!-->\" />";
			return $htmlSearch;
			// return WUI::createViewButton (WLanguage::SEARCH, "search", true);
			*/
			return WUI::createButton (WLanguage::SEARCH, "search", WUI::BUTTON_SUBMIT, array ("extra-class"=>"search_button"));
			
			
		}
			
			
		$btExtra = "";
		if (isset ($props["treeView"])) {
			$treeView = $props["treeView"];
			$btExtra = 
				$treeView->getItemContextButton (WLanguage::MOVE_TO." ...", "MOVE").
				" ".
				$treeView->getItemContextButton (" ▲ ", "MOVE_UP").
				" ".
				$treeView->getItemContextButton (" ▼ ", "MOVE_DOWN");
				
		}

		if (isset ($props["extra_button_func"])) {
			
			$btExtra.=call_user_func ($props["extra_button_func"]);
		}
		
		$btSave = WUI::createButton (WSystem::getArrayValue ($props, "save_text", WLanguage::SAVE), "submit", WUI::BUTTON_SUBMIT, array ("extra-class"=>"edit_button"), true);
		if (isset ($props["no_save"]) && $props["no_save"])
			$btSave = "";
		
		if (isset ($props["no_delete"]) && $props["no_delete"])
			return $btSave." ".$btExtra;

		return WUI::createButton (WSystem::getArrayValue ($props, "delete_text", WLanguage::DELETE), "delete", WUI::BUTTON_DEFAULT, array ("extra-class"=>"delete_button", "style"=>"float:right;", "onclick"=>"WUI.submitFormConfirm('".$this->id."', '".WSystem::rebuildURI ($this->actionURI, array ("cmd"=>$this->id."_delete"))."','".(!isset ($props["deleteQuestion"]) ? WLanguage::QUEST_DELETE_ENTRY : addslashes  ($props["deleteQuestion"]))."');"), false, $btSave." ".$btExtra);
	}	
	
	public function __construct ($id, $cmd="", $actionURI="", $successURI="") {
		parent::__construct ($id);
		if (empty ($actionURI))
			$actionURI = $_SERVER["REQUEST_URI"];

		$this->actionURI = $actionURI;
		$this->successURI = $successURI;
		$this->cmd = $cmd;
	}

	public function __destruct () 
	{
	
	}

}; 



/**

 WFormPage 
 Formular page
 
**/

class WFormPage extends WUIElement
{	
	private $htmlObj = "";
	private $headline = "";
	private $fMsgText = true;
	
	public function __construct ($fMsgText=true) {
		parent::__construct ("page");
		$this->fMsgText = $fMsgText;
	}
	
	public function __destruct () {
	
	}
	
	public function addObject ($obj) {
		$this->htmlObj.=$obj;
	}
	
	public function setHeadline ($headline) {
		$this->headline = $headline;
	}
	
	public function renderElement () {

		// Default scripts
		$focusFirstInput = 			
			'
			<script type="text/javascript">
				
				var focusId="'.WUI::getFocusId ().'";
				if (focusId)
					jQuery(\'#\'+focusId).focus();			
				else
					jQuery(\'input:text:visible:enabled:first\').focus();
				
				jQuery (document).ready (function () { 
					WUI.sessionLogFilename=\''.SERVER_URL.'/tmp/'.session_id().'.log\'; 
				});
			</script>			
			';
			
		$html = '';
		// Headline
		if (!empty ($this->headline))
			$html.='<h1><!-->'.htmlspecialchars ($this->headline).'<!--></h1>';

		// Progressbar
		$html.='
			<div id="page_progress" style="display:none;"></div>';

		// Message
		if ($this->fMsgText)
			$html.='<div class="msgText" id="msgText" style="display:none;"></div>';

		// Content
		$html.=$this->htmlObj;

		// Show
		if ((int) WSystem::getParam ($this->id."_is_ajax")) {
			$this->html = $html;
			WUI::updateAreaAsync ("wrap_".$this->id, $html);
			
			// Check style theme
			global $session;
			if ($session->isLogin ()) {
				$themeCheck = "";
				if (isset ($_SESSION["style_theme"]))
					$themeCheck = $_SESSION["style_theme"];				
				if ($session->getStyleTheme ()!=$themeCheck) {		
					$focusFirstInput.=$session->setStyleTheme ($session->getStyleTheme (), 3);
				}
			}
			
			WUI::command (array ("evalScript" => $focusFirstInput));
		}

		$this->html = 
			'
			<div id="wrap_'.$this->id.'">'.
					$html.
			'</div>
			 <div id="popup_filter"></div>
			 <div id="no_select_filter"></div> 			 
			 '.
			$focusFirstInput;
			
		
	}

}


/**

 WFormUtils
 Input parsing

**/

class WFormUtils
{
	public static function readInput ($fields, &$values, $fInitDefauls=false) {
		$errCnt = 0;
		foreach ($fields as $name => $tmp) 
		{
			$list = explode (';', $tmp);
			if (count ($list)<2)
				continue;
			
			$type       = $list[0];
			$mode       = (count ($list)>1) ? $list[1] : "*";
			$default    = (count ($list)>2) ? $list[2] : WFormat::getEmptyValue ($type);
			$value      = (isset ($_POST[$name])) ? trim (stripslashes($_POST[$name])) : "";
			$errText    = "";
			$fMandatory = 0;

			if ($mode=="$") // Dont read any input
				continue;
			
			if ($fInitDefauls) {
				$value = $default;
			}
			else {

				// Check if input value is empty
				$fAvailable = isset ($_POST[$name]) || empty ($value);
				if (!$fAvailable && $mode=='m') {
					$fMandatory = 1;
					$value = WFormat::getEmptyValue ($type);
				}
				else {
		
					if (WFormat::check ($value, $type, $errText)) {				
						// Check if parsed value is empty
						$value = WFormat::parse ($value, $type, $errText);
						if ($value==WFormat::getEmptyValue ($type) && !($type=="file" && isset($_POST[$name."_tmp"]) && !empty($_POST[$name."_tmp"]))) {


							if ($mode=='m')
								$fMandatory = 1;
						}
					}
				}
			}
			
			$fError = !empty ($errText);
			if ($fError || $fMandatory) {
				$errCnt++;
				// echo $name.'='.$value.' '.$errText.'<br />';

			}
			$values[$name]=array (
				"type"        => $type,
				// Parsed value or empty
				"val"         => $fError ? WFormat::getEmptyValue ($type) : $value,
				// Formatted value or original
				"valf"        => $fError ? $value : WFormat::format ($value, $type),
				"isMandatory" => $fMandatory,
				"isError"     => $fError,
				"errText"     => $fMandatory ? WLanguage::FIELD_REQUIRED : $errText
			);

		}

		return $errCnt;
	}
	
	public static function initInput ($fields, &$values, $data) {
		foreach ($fields as $name => $tmp) 
		{
			$list = explode (';', $tmp);
			if (count ($list)<2)
				continue;
			
			$type     = $list[0];
			$mode     = (count ($list)>1) ? $list[1] : "*";
			$default  = (count ($list)>2) ? $list[2] : WFormat::getEmptyValue ($type);
			$value    = (count ($list)>2) ? $list[2] : WFormat::getEmptyValue ($type);
						
			if ($data && isset ($data[$name]))
				$value = $data[$name];
			else
				$value = $default;
				
			$values[$name]=array (
				"type"        => $type,
				"val"         => $value,
				"valf"        => $value,
				"isMandatory" => ($mode=='m'),
				"isError"     => 0,
				"errText"     => ""
			);
		}
	} 	
	
	public static function getFormattedInputValues (&$values) {
		$values2 = array ();
		foreach ($values as $name => $data) {
			$values2[$name]=$data["valf"];
		}
		return $values2;
	}	
	
	public static function getParsedInputValues (&$values) {
		$values2 = array ();
		foreach ($values as $name => $data) {
			$values2[$name]=$data["val"];
		}
		return $values2;
	}	
};

?>