<?php

/**

 WFormat
 Formatting, parsing, checking datatypes

**/


class WFormat 
{
	/* Native types */
	const NATIVE_TYPE_STRING   = 1;
	const NATIVE_TYPE_INT      = 2;
	const NATIVE_TYPE_FLOAT    = 3;
	const NATIVE_TYPE_BOOL     = 4;
	const NATIVE_TYPE_DATE     = 5;
	const NATIVE_TYPE_DATETIME = 6;

			
	private static $typeList = array (
		/* Basic types */
		"s" 					=> array (self::NATIVE_TYPE_STRING,   255,  0, "VARCHAR(255)"),
		"i" 					=> array (self::NATIVE_TYPE_INT,        9,  0, "INT"),
		"f" 					=> array (self::NATIVE_TYPE_FLOAT,     12,  6, "FLOAT(12,5)"),
		"c" 					=> array (self::NATIVE_TYPE_FLOAT,     12,  2, "FLOAT(12,2)"),	 // Currency
		"b" 					=> array (self::NATIVE_TYPE_BOOL,       1,  0, "TINYINT(1)"),
		"d" 					=> array (self::NATIVE_TYPE_DATE,      10,  0, "DATE"),
		"t" 					=> array (self::NATIVE_TYPE_DATETIME,  19,  0, "TIMESTAMP"),

		/* Application types */
		"text_small"  => array (self::NATIVE_TYPE_STRING,    32,  0, "VARCHAR(32)"),
		"text_medium" => array (self::NATIVE_TYPE_STRING,   127,  0, "VARCHAR(127)"),
		"text_long"   => array (self::NATIVE_TYPE_STRING,  4095,  0, "VARCHAR(4095)"),
		"text_big"    => array (self::NATIVE_TYPE_STRING,    -1,  0, "TEXT"),
		"email"       => array (self::NATIVE_TYPE_STRING,   127,  0, "VARCHAR(127)"),
		"filename"    => array (self::NATIVE_TYPE_STRING,   255,  0, "VARCHAR(255)"),
		"url"         => array (self::NATIVE_TYPE_STRING,   255,  0, "VARCHAR(255)"),
		"www"         => array (self::NATIVE_TYPE_STRING,   255,  0, "VARCHAR(255)"),
		"i0"  			  => array (self::NATIVE_TYPE_INT,        9,  0, "INT"),           // Int not show zero
		"f0" 					=> array (self::NATIVE_TYPE_FLOAT,     12,  6, "FLOAT(12,5)"),	 // Float not show zero
		"c_no_dot"  	=> array (self::NATIVE_TYPE_FLOAT,     12,  2, "FLOAT(12,2)"),	 // Currency not show thousands separator

	);
	
	
	public static function getSQLType ($type) {
		if (isset (self::$typeList[$type]))
			return self::$typeList[$type][3];
		return "VARCHAR(255)";	
	}
	
	public static function check ($value, $type, &$errText) 
	{
		$ret = true;
		$errText = "";

		if (!isset (self::$typeList[$type])) {
			return true; // No check
		}		
		
		$typeData = self::$typeList[$type];
		
		/* Application types */		
		switch ($type) 
		{
			case "email":

				if ($value!="" && !preg_match ("/^([a-zA-Z0-9\.\_\-]+)@([a-zA-Z0-9\.\-]+\.[A-Za-z][A-Za-z]+)$/", $value)) {
				// if ($value!="" && !filter_var($value, FILTER_VALIDATE_EMAIL)) {									
					$errText = WLanguage::INVALID_EMAIL;
					$ret = false;
				}
				break;
				
			case "filename":
				if ($value!="" && !preg_match ("/^[a-zA-Z0-9._-öuäÖÜÄ-]*$/", $value)) {
					$errText = WLanguage::INVALID_FILENAME;
					$ret = false;
				}
				break;

			case "www":
			case "url":

				$regex = "#^http(s)?://[a-z0-9-_.]+\.[a-z]{2,4}#i";
				/*
				$regex = "((https?|ftp)\:\/\/)?"; // SCHEME
		    $regex .= "([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?"; // User and Pass
		    $regex .= "([a-z0-9-.]*)\.([a-z]{2,3})"; // Host or IP
		    $regex .= "(\:[0-9]{2,5})?"; // Port
		    $regex .= "(\/([a-z0-9+\$_-]\.?)+)*\/?"; // Path
		    $regex .= "(\?[a-z+&\$_.-][a-z0-9;:@&%=+\/\$_.-]*)?"; // GET Query
		    $regex .= "(#[a-z_.-][a-z0-9+\$_.-]*)?"; // Anchor
				*/
				
				
				if ($value!="" && !preg_match ($regex, $value)) {
					$errText = WLanguage::INVALID_URL;
					$ret = false;
				}
				break;
				
			case "i0":
				
				break;
			case "f0":
				
				break;
		}
		
		/* Native types */
		switch ($typeData[0])
		{
			case self::NATIVE_TYPE_DATE:
			case self::NATIVE_TYPE_DATETIME:
				if (!WLanguage::checkDate($value)) {
					$errText = WLanguage::INVALID_DATE;
					$ret = false;
				}
				break;
				 
			case self::NATIVE_TYPE_FLOAT: 
				$value = str_replace (",", ".", $value);
				if ($value!="" && !preg_match("/^([+-]?[0-9]*[.,]?[0-9]+)$/", $value)) {
					$errText = WLanguage::INVALID_FLOAT;
					$ret = false;
				}
				else
				if (strlen (abs (floor (floatval ($value))))>$typeData[1])  {							
					$errText = sprintf (WLanguage::FLOAT_TO_LONG, $typeData[1]);
					$ret = false;							
				}
				break;

			case self::NATIVE_TYPE_INT: 
				if ($value!="" && !preg_match("/^([+-]?[0-9]+)$/", $value)) {
					$errText = WLanguage::INVALID_FLOAT;
					$ret = false;
				}
				else
				if ($value!="" && !preg_match("/^([+-]?[0-9]{0,".$typeData[1]."}+)$/", $value)) {
					$errText = sprintf (WLanguage::INT_TO_LONG, $typeData[1]);
					$ret = false;
				}
				break;
				
			case self::NATIVE_TYPE_STRING:
				if ($typeData[1]!=-1 && strlen ($value)>$typeData[1]) {
					$errText = sprintf (WLanguage::STRING_TO_LONG, $typeData[1]);
					$ret = false;
				}
				break;
		}
		return $ret;
	}
		
	public static function parse ($value, $type, &$errText) 
	{
		$errText = "";
		$ret = $value;

		if (!self::check ($value, $type, $errText))
			return self::getEmptyValue ($type);

		if (isset (self::$typeList[$type])) {
			$typeData = self::$typeList[$type];
			switch ($type) 
			{		
				/* Application types */
				case "email":
					$ret = strtolower ($value);
					break;
				
				case 'www':
					$ret = self::formatWebsiteLink ($value, true);
					break;	
					

				
				/* Native types */		
				default:			
					switch ($typeData[0]) {
					
						case self::NATIVE_TYPE_DATE:
							$ret = WLanguage::parseDate ($value);
							break;
							
						case self::NATIVE_TYPE_DATETIME:
							$ret = WLanguage::parseDateTime ($value);
							break;
							
						case self::NATIVE_TYPE_FLOAT:
							$ret = str_replace (",", ".", round (floatval (str_replace (",", ".", $value)), $typeData[2])); 

							break;
							
						case self::NATIVE_TYPE_INT:
							$ret = intval ($value); 
							break;

						case self::NATIVE_TYPE_BOOL:
							$ret = (strtolower ($value)=="on" || $value==1 || strtolower ($value)=="true") ? 1 : 0; 
							break;
					}
					break;
			}
		}
		return $ret;
	}

	public static function format ($value, $type, $fUI=false, $flag=0) 
	{
		$ret = $value;
		if (!$fUI && !isset (self::$typeList[$type]))
			return $ret;
			
		if ($type=="t2") {
			$type="t"; $flag = 1; // Time only with minutes
		}

		$typeData = self::$typeList[$type];
		switch ($type) 
		{		
			/* Application types */
			case 'c':
				$ret = self::formatCurrency ($value, $fUI);
				break;
				
			case 'f':
				$ret = self::formatNumber ($value);
				break;
				
			case 'www':
				$ret = self::formatWebsiteLink ($value, !$fUI);
				break;
			case "f0":
				if ($value!=0)
					return self::formatNumber ($value);;
				return "";
				break;
			
			case "i0":
				if (!empty ($value))
					return $value;
				return "";
				break;
				
			case "kb": 
				$ret = self::formatKBText ($value);
			 	break;

			/* Native types */
			default:
				switch ($typeData[0]) {
					case self::NATIVE_TYPE_DATE:
						if ($value=="0000-00-00" || $value=="0000-00-00 00:00:00") {
							$ret = "";
						}
						else
						if (sscanf ($value, "%04d-%02d-%02d", $year, $month, $day)>=3) {
							$ret = WLanguage::formatDate ($year, $month, $day);
						}	
						break;
					
					case self::NATIVE_TYPE_DATETIME:

						if ($value=="0000-00-00 00:00:00") {
							$ret = "";
						}
						else
						if (sscanf ($value, "%04d-%02d-%02d %02d:%02d:%02d", $year, $month, $day, $hour, $minute, $second)==6) {				
							if ($flag)
								$second=-1; 
							$ret = WLanguage::formatDateTime($year, $month, $day, $hour, $minute, $second);
						}
						break;
						
					case self::NATIVE_TYPE_FLOAT:
						$ret = str_replace (".", ",", round ($value, $typeData[2]));
						break;
				}
				break;
		}
		return $ret;
	}
	
	public static function getEmptyValue ($type, $fNative=false) {
		$ret = "";
		if (isset (self::$typeList[$type])) {
			$typeData = self::$typeList[$type];
			switch ($type) 
			{		
				/* Application types */
				case "i0":
				case "f0":
					if (!$fNative) {
						return "";
						break;
					}
									
				/* Native types */		
				default:			
					switch ($typeData[0]) {					
						case self::NATIVE_TYPE_DATE:
							$ret = "0000-00-00";
							break;

						case self::NATIVE_TYPE_DATETIME:
							$ret = "0000-00-00 00:00:00";
							break;

						case self::NATIVE_TYPE_INT:							
						case self::NATIVE_TYPE_FLOAT:
						case self::NATIVE_TYPE_BOOL:
							$ret = 0;
							break;
							
					}
					break;
			}
		}
		return $ret;
	} 	
	
	/* Helpers */	
	public static function formatCurrency ($value, $fSepThousand=true, $fWithSymbol=false, $precision=2) 
	{	
		if (empty ($value))
			$value = 0;
		$str = sprintf("%01.".$precision."f", round ($value, $precision));                
		$str2 = "";
		if ($fSepThousand)
			$str2 = number_format($value, $precision, ',', '.');
		else
			$str2 = str_replace(".", ",", $str);
		if ($fWithSymbol)
			$str2.=" ".WSystem::getConf ("currency_symbol", WLanguage::CURRENCY_SYMBOL);
		return $str2;
	}
	
	public static function formatDate ($value) {
		return self::format ($value, "d");
	}
	
	public static function formatDateTime ($value, $fOnyMinutes=false) {
		return self::format ($value, "t", false, $fOnyMinutes ? 1 : 0);
	}
	
	public static function formatNumber ($value, $precision=5) 
	{
		return str_replace (".", ",", round (floatval ($value), $precision));
	}
	
	public static function formatKBText ($sizeBytes) {
		if ($sizeBytes>=(1048576*1024))
			return round ($sizeBytes / (1048576*1024), 1)." GB";
		else
		if ($sizeBytes>=1048576)
			return round ($sizeBytes / 1048576, 1)." MB";
		else	
		if ($sizeBytes>=1024)
			return round ($sizeBytes / 1024)." KB";		
		else
			return $sizeBytes." Byte";		
	}
	
	public static function formatWebsiteLink ($value, $fOnlyLink=false) {
		if ($value=="")
			return "";
		// Check email
		if (strpos ($value, "@")!==false) {
			if (!$fOnlyLink)
				return "<a href=\"mailto:".$value."\">".$value."</a>"; 
			else
				return "mailto:".$value;
		}			
		// Check URL
		$protocolList = array ("http://", "https://", "ftp://");
		$found = false;
		foreach ($protocolList as $protocol) {
			$pos = strpos ($value, $protocol);
			if ($pos !== false && $pos==0)
				$found = true;
		}
		if (!$found) {
			$pos = strpos ($value, "www");
			if ($pos === false || $pos!=0)
				$link="http://www.".$value;
			else
				$link="http://".$value;	
		}
		else
			$link=$value;
		if (!$fOnlyLink)
			return "<a href=\"".$link."\" target=\"_blank\">".$value."</a>"; 
		return $link;
	}
	
	public static function formatCSVText ($text, $maxChars=0) {
		$str = str_replace("\r\n", " ", $text); // replace line feeds
		$str = str_replace("\n", " ", $str); // replace line feeds
		if ($maxChars>0)
			$str = substr ($str, 0, $maxChars);
		return "\"".str_replace("\"", "\"\"", utf8_decode ($str))."\"";
	}	
	
	
	public static function date2TimeValue ($date) {
		if (count ($list = sscanf($date, "%04d-%02d-%02d %02d:%02d:%02d"))==6) {		
			list($year, $month, $day, $hour, $minute, $second) = $list;
			return (int) mktime($hour, $minute, $second, $month, $day, $year);
		}
		else
		if (count ($list = sscanf($date, "%04d-%02d-%02d"))==3) {		
			list($year, $month, $day) = $list;
			return (int) mktime(0, 0, 0, $month, $day, $year);
		}
		return 0;
	}
	
	public static function timeValue2Date ($time, $fOnlyDate=false) {
		if ($fOnlyDate)
			return strftime("%Y-%m-%d", $time);
		return strftime("%Y-%m-%d %H:%M:%S", $time);
	}

	public static function compareDates ($dateStart, $dateStop) 
	{
		$time = self::date2TimeValue ($dateStart);
		$time2 = self::date2TimeValue ($dateStop);
		if ($time2<$time && $time2!=0)
			return false;
		return true;
	}	

};

?>