<?php

/**

 WImageUtils

**/

abstract class WImageUtils
{
	const SCALE_AUTO = 0;
	const SCALE_X = 1;
	const SCALE_Y = 2;
	const SCALE_XY = 3;	
	const SCALE_XY_MARGIN = 4;	
		
	public static function createThumbnail ($srcFilename, $dstFilename, $thumbWidth, $thumbHeight, $scaleMode=WImageUtils::SCALE_AUTO, $quality=95, $bkg=array (255,255,255), $fForce=false) {	
		$fRet = true;
		$target_id = 0;
		
		$ext = strtolower (WUtils::getFileExtension ($srcFilename));
		switch ($ext) {
			case "jpg":
				$source_id = imageCreateFromJPEG($srcFilename); 
				break;
			case "png":
				$source_id = imageCreateFromPNG($srcFilename); 
				break;
			case "gif":
				$source_id = imageCreateFromGIF($srcFilename); 
				break;
			default:
				return false;
				break;
		};
		
		if (!$source_id)
			$fRet = false;
		else {
			$picsize = getimagesize($srcFilename);
			if ($picsize) {
				$src_width  = $picsize[0];
				$src_height = $picsize[1];
				
				if ($src_width<=$thumbWidth && $src_height<=$thumbHeight && ($scaleMode!= self::SCALE_XY_MARGIN && $scaleMode!= self::SCALE_XY))
					$fRet = false;
				
			}
			else
				$fRet = false;
			if ($fRet || $fForce) {
				// Calc best resolution for thumbnail                    
				$mx = $thumbWidth  / $src_width;
				$my =   $thumbHeight / $src_height;
				
				
				switch ($scaleMode) {				
					case self::SCALE_AUTO:
					case self::SCALE_XY_MARGIN:
						if ($mx<$my) 
							$m=$mx; 
						else 
							$m=$my;
						break;
					case self::SCALE_X:					
					case self::SCALE_XY:					
						$m=$mx; 					
						break;
					default:					
						$m=$my;
						break;
				}
				$dst_width  = round ($m * $src_width);
				$dst_height = round ($m * $src_height);
				if (self::SCALE_XY_MARGIN) {
					if ($dst_width>$src_width || $dst_height>$src_height) {
						$dst_width = $src_width;
						$dst_height = $src_height;
					}
				}

	
				if ($scaleMode==self::SCALE_XY) {


					$m = $thumbWidth/$thumbHeight;
					$src_height2 = $src_height;
					$src_width2 = $src_height*$m;
					if ($src_width2>$src_width) {
						$m = $thumbHeight/$thumbWidth;
					
						$src_width2 = $src_width;
						$src_height2 = $src_width*$m;
					}
				
					
					
					$src_width2  =(int) round($src_width2);	
					$src_height2 =(int) round($src_height2);	

					$target_id  = imagecreatetruecolor ($thumbWidth, $thumbHeight);
					$white = imagecolorallocate($target_id, 255, 255, 255);
					imagefill($target_id, 0, 0, $white);
					
					$target_pic = imagecopyresampled ($target_id, $source_id,0,0,round (($src_width-$src_width2)/2),round (($src_height-$src_height2)/2),$thumbWidth,$thumbHeight,$src_width2,$src_height2);
				}
				else
				if ($scaleMode==self::SCALE_XY_MARGIN) {
					
					$target_id  = imagecreatetruecolor ($thumbWidth, $thumbHeight);
					$white = imagecolorallocate($target_id, $bkg[0], $bkg[1], $bkg[2]);
					imagefill($target_id, 0, 0, $white);
					$target_pic = imagecopyresampled ($target_id, $source_id,round (($thumbWidth-$dst_width)/2),round (($thumbHeight-$dst_height)/2),0,0,$dst_width,$dst_height,$src_width,$src_height);
				}
				else {
					// Resample
					$target_id  = imagecreatetruecolor ($dst_width, $dst_height);
					$white = imagecolorallocate($target_id, 255, 255, 255);
					imagefill($target_id, 0, 0, $white);
					
					$target_pic = imagecopyresampled ($target_id, $source_id,0,0,0,0,$dst_width,$dst_height,$src_width,$src_height);
		
				}
				
		

			
				if ($target_pic) { 
					if (is_file ($dstFilename))
						unlink ($dstFilename);
					
					switch ($ext) {
						case "jpg":
							imagejpeg ($target_id, $dstFilename, WSystem::getConf ("img_quality", $quality));						
							break;
						case "png":
							imagepng ($target_id, $dstFilename);							
							break;
						case "gif":
							imagegif ($target_id, $dstFilename);	
							break;
					};
									
					$fRet = true;
				}
			}
		}
		if ($source_id)
			imagedestroy ($source_id);
		if ($target_id)
			imagedestroy ($target_id);
		return $fRet;
	}
}

?>