<?php

/**

 WSystem
 System functions, utils

**/

function g_stripslashesDeep ($value) {
	if (function_exists("get_magic_quotes_gpc") && get_magic_quotes_gpc())
		$value = is_array($value) ? array_map("g_stripslashesDeep", $value) : stripslashes($value);
	return $value;
}

	
class WSystem
{
	private static $requestURI = "";
	private static $logFunc = NULL;
	
	public static function isParam ($name) {
		return isset ($_GET[$name]) || isset ($_POST[$name]);
	}
	
	public static function getParam ($name, $default="", $fOnlyGet=false) {
		if (isset ($_GET[$name]))		
			return g_stripslashesDeep($_GET[$name]);
		if (!$fOnlyGet) {
			if (isset ($_POST[$name]))
				return g_stripslashesDeep($_POST[$name]);
		}
		return $default;
	}
	
	public static function getParamFromSession ($name, $default="") {
		$value=$default;
		if (!self::isParam ($name)) {		
			if (isset ($_SESSION[$name]))
				$value = $_SESSION[$name];
		}
		else
			$_SESSION[$name]=($value=self::getParam ($name));
		return $value;
	}
	
	public static function getArrayValue ($array, $key, $default="", $subKey="") {
		if (isset ($array[$key])) {
			if ($subKey!="") {	
				if (isset ($array[$key][$subKey]))
					return $array[$key][$subKey];
				else
					return $default;
			}
			return $array[$key];		
		}
		return $default;	
	}
	
	public static function getConf ($key, $default="", $fLoadFromDB=false) {
		global $CONFIG_GLOBAL, $wdb;
		if (isset ($CONFIG_GLOBAL[$key])) 
			return $CONFIG_GLOBAL[$key];
			
		if ($fLoadFromDB) {
			$_SESSION[SESSION_PREFIX."CONFIG_ALL"][$key] = $wdb->getId ("select value from ".DB_TABLE_PREFIX."property where name='".WDB::escape ($key)."'", "value");
		}					
		return self::getArrayValue ($_SESSION, SESSION_PREFIX."CONFIG_ALL", $default, $key);	
	}
	
	public static function getConfList ($key, $default="", $fLoadFromDB=false) {
		$value = self::getConf ($key, $default, $fLoadFromDB);
		if ($value=="")
			$value = $default;
		$confList = array ();
		$list = explode (";", $value);										
		foreach ($list as $token) {
			$list2 = explode ("=", $token);
			if (count ($list2)==2)
				$confList[$list2[0]]=$list2[1];
			else
				$confList[$list2[0]]=$list2[0];
		}
		return $confList;
	}
	
	public static function setConf ($key, $value) {
		global $wdb;
		if ($wdb) {
			$sql = "replace into ".DB_TABLE_PREFIX."property (value,name) values ('".WDB::escape ($value)."','".WDB::escape ($key)."')";
			$wdb->query ($sql);
			$_SESSION[SESSION_PREFIX."CONFIG_ALL"][$key]=$value;
		}
	}
	
	public static function loadConf ($fRefresh=false) {
		global $wdb, $CONFIG_GLOBAL;
		if ($fRefresh) {
			unset ($_SESSION[SESSION_PREFIX."CONFIG_ALL"]);
			unset ($_SESSION[SESSION_PREFIX."isConfigFromDBLoaded"]);
		}

		if (!isset ($_SESSION[SESSION_PREFIX."CONFIG_ALL"]))
			$_SESSION[SESSION_PREFIX."CONFIG_ALL"]=$CONFIG_GLOBAL;

		if ($wdb) {
			if (!isset ($_SESSION[SESSION_PREFIX."isConfigFromDBLoaded"])) {
				$wdb->query ("select * from ".DB_TABLE_PREFIX."property");
				while ($wdb->fetch ($dbRow)) {
					$_SESSION[SESSION_PREFIX."CONFIG_ALL"][$dbRow["name"]]=$dbRow["value"];
				}
				$_SESSION[SESSION_PREFIX."isConfigFromDBLoaded"]=true;
			}
		}		
		// Unset important config vars
		unset ($_SESSION[SESSION_PREFIX."CONFIG_ALL"]["db_password"]);
	}
	
	public static function rebuildURI ($uri, $params=NULL, $fOnlyFirstParam=false, $ignoreParams=NULL) {			
		$pos = strpos ($uri, '?');

		if ($pos===false) {
			// No params
			$ext2 = "";
			$base = $uri;
		}
		else
		{
			$base = substr ($uri, 0, $pos);	
			$ext  = substr ($uri, $pos+1, strlen ($uri)-$pos);			
			$ext2 = "";
			$list = explode ('&', $ext);

			foreach ($list as $token) {

				list ($key, $value) = explode ('=', $token);
				if ($ignoreParams && (array_key_exists ($key, $ignoreParams) || in_array ($key, $ignoreParams)))
					continue;
					
				if (!$params || !isset ($params[$key])) {
					if (empty ($ext2))
						$ext2.='?';
					else
						$ext2.='&';
					$ext2.=($key."=".$value);

				}
				if ($fOnlyFirstParam)
					break;

			}
		}

		if ($params) {
			foreach ($params as $key => $value) {

				if (empty ($ext2))
					$ext2.='?';
				else
					$ext2.='&';
				$ext2.=($key."=".$value);
			}
		}
		$uri = $base.$ext2;
		
		return $uri;
	}
	
	public static function setLogFunction ($logFunc) {
		self::$logFunc = $logFunc;
	}
	
	public static function log ($log, $fReset=false) {
		if (self::$logFunc) {
			call_user_func (self::$logFunc, $log, $fAppend);
			return;
		}
		
		$sessionLogFilename = ROOT_PATH."/tmp/".session_id().".log";
		$fh = fopen ($sessionLogFilename, (!$fReset ? "a+" : "w+"));
		if ($fh) {
			fwrite ($fh, htmlspecialchars ($log).'<br />');
			fclose ($fh);
		}
	}
	
	public static function clearLog () {
		$sessionLogFilename = ROOT_PATH."/tmp/".session_id().".log";
		if (is_file ($sessionLogFilename)) {
			unlink ($sessionLogFilename);
		}
	}
	
	public static function checkConnection () {
		
		header ("Content-Type: text/plain");
		echo " ";
		flush ();
		if (connection_aborted()) {				
			exit;
		}

	}

	public static function updateRequestURI ($params=NULL, $fOnlyFirstParam=false, $ignoreParams=NULL) {
		self::$requestURI = $_SERVER["REQUEST_URI"] = self::getRequestURI ($params, $fOnlyFirstParam, $ignoreParams);
	}
	
	public static function getRequestURI ($params=NULL, $fOnlyFirstParam=false, $ignoreParams=NULL) {			
		$uri = empty (self::$requestURI) ? $_SERVER["REQUEST_URI"] : self::$requestURI;
		return self::rebuildURI ($uri, $params, $fOnlyFirstParam, $ignoreParams);
	}
	
	public static function utf8_strtolower ($str) {
		return utf8_encode(strtolower (utf8_decode($str)));
	}

	public static function utf8_strtoupper ($str) {
		return utf8_encode(strtoupper (utf8_decode($str)));
	}	
	
	public static function createHTMLException ($e) {
		header ("Content-Type: text/html");

		date_default_timezone_set ("Europe/Berlin");

		$html = 
		"<table class=\"exception\">".
			"<tr><td colspan=\"2\"><b>Exception</b></td></tr>".
			"<tr><td>Time</td><td>".date("Y-m-d H:i:s")."</td></tr>".
			"<tr><td>File</td><td>".$e->getFile ()."</td></tr>".
			"<tr><td>Line</td><td>".$e->getLine ()."</td></tr>".
			"<tr><td>Message</td><td>".$e->getMessage ()."</td></tr>".
			"<tr><td>Code</td><td>".$e->getCode ()."</td></tr>".
			// "<tr><td>Trace</td><td>".$e->getTraceAsString  ()."</td></tr>".
		"</table>";
		print ($html);
		exit;

	}
	
};

?>
