<?php



// TreeDB
class TreeDB {

	private $columnMap = array (
		"item_id"      => "tree_id",
		"parent_id"    => "parent_tree_id",
		"name"         => "name",
		"sort_pos"     => "sort_pos",
		"show_open"    => "has_children"
	);
	
	private $table = "tree";
	
	private $wdb = NULL;
	private $whereClause = "";
	
	public function mapValues (&$values) {
		$values2 = array ();
		foreach ($values as $key => $value) {
			if (isset ($this->columnMap[$key])) {
				$values2[$this->columnMap[$key]]=$value;
			}
		} 
		return $values2;
	}
	
	public function remapValues (&$values) {
		$flipMap = array_flip ($this->columnMap);
		$values2 = array ();
		foreach ($values as $key => $value) {
			if (isset ($flipMap[$key])) {
				$values2[$flipMap[$key]]=$value;
			}
		} 
		if ($this->columnMap["name"]!="name")
			$values2[$this->columnMap["name"]]=$values2["name"];
		
		return $values2;
	}
	
	public function __construct ($wdb, $table=NULL, $columnMap=NULL, $whereClause="") {	
		$this->wdb = $wdb;
		if ($table)
			$this->table = $table;
		if ($columnMap) {
			$this->columnMap = array_merge ($this->columnMap, $columnMap);
		}
		$this->whereClause=$whereClause;
	}
	
	public function insertItem ($values) {
		$itemId2 = $values["item_id"];
		unset ($values["item_id"]);
		$values2 = $this->mapValues ($values);


		$itemId = $this->wdb->insert ($this->table, $values2);
		
		$parentId = $values["parent_id"];	
		if ($parentId)
			$this->wdb->update ($this->table, array ($this->columnMap["show_open"] => 1), $this->columnMap["item_id"], $parentId);				
		return $itemId;
	}
	
	public function updateItem ($values) {
		$itemId = $values["item_id"];
		$values2 = $this->mapValues ($values);
		
		// Check if has children
		$sql = 'select count(*) as cnt from '.$this->table.' where is_deleted=0 and '.$this->columnMap["parent_id"].'='.$itemId;
		if ($this->wdb->getId ($sql, "cnt")) {
			$values2[$this->columnMap["show_open"]]=1;
		}
		else {
			$values2[$this->columnMap["show_open"]]=0;		
		}
		return $this->wdb->update ($this->table, $values2, $this->columnMap["item_id"], $itemId);
	}	
		
	public function removeItem ($itemId) {	
		// Get parent item id
		$sql = 'select '.$this->columnMap["parent_id"].' from '.$this->table.' where '.$this->columnMap["item_id"].'='.$itemId;
		
		$parentId = $this->wdb->getId ($sql, $this->columnMap["parent_id"]);
		
	 	$ret = $this->wdb->delete ($this->table, $this->columnMap["item_id"], $itemId);
	 	if ($ret) {		
			if ($parentId) {
				// Check if parent has children
				$sql = 'select count(*) as cnt from '.$this->table.' where '.$this->columnMap["parent_id"].'='.$parentId.' and is_deleted=0';
				if (!$this->wdb->getId ($sql, "cnt"))
					$this->wdb->update ($this->table, array ($this->columnMap["show_open"] => 0), $this->columnMap["item_id"], $parentId);				
			}
			return true;
		}
		return false;
	}
	
	public function updateItemAttribute ($itemId, $attr, $value) {
		$sql = 'update '.$this->table.' set '.$this->columnMap[$attr].'=\''.WDB::escape ($value).'\' where '.$this->columnMap["item_id"].'='.$itemId;
		return $this->wdb->query ($sql);
	}
	
	public function queryItems ($parentId) {
		$sql = 'select * from '.$this->table.' where '.$this->columnMap["parent_id"].'='.$parentId.'';
		if (!empty ($this->whereClause))
			$sql.=' and '.$this->whereClause;		
		$sql.=' order by sort_pos';
		return $this->wdb->query ($sql);
	}
	
	public function fetchItem (&$values) {
		$dbRow = array ();
		if ($this->wdb->fetch ($dbRow)) {
			$values =  $this->remapValues ($dbRow);
			return true;
		}
		return false;
	}

};

/**

 WTree
 Tree data handling

**/

class WTree implements Serializable
{
	protected $idMax = 0;
	protected $itemList = array ();
	protected $parentItemList = array ();
	protected $selectedItemId = 0;
	protected $treeDB = NULL;
	protected $ignoreItemId = 0;
	
	public function __construct ($treeDB=NULL) {
		$this->treeDB = $treeDB;

	}
	
	public function __destruct () {
	
	}
	
	
	public function serialize() {
		return serialize (
			array (
				"idMax" => $this->idMax,
				"itemList" => $this->itemList,
				"parentItemList" => $this->parentItemList,
				"selectedItemId" => $this->selectedItemId,
				"ignoreItemId" => $this->ignoreItemId));
  }
  
  public function reset () {
		$this->idMax=0;
		$this->itemList = array ();
		$this->parentItemList = array ();
		$this->selectedItemId = 0;
		$this->ignoreItemId = 0;
	
	}
  
  public function unserialize ($data) {
  	$values = unserialize($data);
  	$this->idMax = $values["idMax"];
  	$this->itemList = $values["itemList"];
  	$this->parentItemList = $values["parentItemList"];
  	$this->selectedItemId = $values["selectedItemId"];
  	$this->ignoreItemId = $values["ignoreItemId"];
  }
  
  public function initFromDB () {
		if (!$this->treeDB)
			return false;
		$this->idMax = 0;
		$this->itemList = array ();
		$this->parentItemList = array ();
		$this->selectedItemId = 0;
					
		$this->treeDB->queryItems (0);
		$itemData = array ();
		while ($this->treeDB->fetchItem ($itemData)) {
			$itemId = $itemData["item_id"];
			$parentId = $itemData["parent_id"];
			$this->itemList[$itemId]=$itemData;
			$this->parentItemList[$parentId][$itemId]=&$this->itemList[$itemId];
			if ($itemId>$this->idMax)
				$this->idMax=$itemId;
		}
		return true;
	}
	
	public function openItem ($parentId) {
		if ($this->treeDB) {
			$this->treeDB->queryItems ($parentId);
			while ($this->treeDB->fetchItem ($itemData)) {
				$itemId = $itemData["item_id"];
				$parentId = $itemData["parent_id"];
				$this->itemList[$itemId]=$itemData;
				$this->parentItemList[$parentId][$itemId]=&$this->itemList[$itemId];
				if ($itemId>$this->idMax)
					$this->idMax=$itemId;
			}
		}
		$this->updateItemAttribute ($parentId, "is_open", 1);
	}
	
	public function closeItem ($parentId) {
		$this->updateItemAttribute ($parentId, "is_open", 0);
		
		// $tree->removeChildren($itemId);
	}
	
	public function setIgnoreItemId ($itemId) {
		$this->ignoreItemId = $itemId;		
	}
	
	public function getIgnoreItemId () {
		return $this->ignoreItemId;
	}
	
	private function getMaxSortPos ($parentId) {
		if (!isset ($this->parentItemList[$parentId]))
			return 0;
		$itemList = $this->parentItemList[$parentId];
		$maxSortPos = 0;
		foreach ($itemList as $itemId => $itemData) {
			if ($itemData["sort_pos"]>$maxSortPos)
				$maxSortPos = $itemData["sort_pos"];
		}
		return $maxSortPos;
	}

	public function isBottomItem ($itemId) {
		if (!isset ($this->itemList[$itemId]))
			return false;
		$fBottom = false;
		$parentId = $this->itemList[$itemId]["parent_id"];
		$itemList = $this->parentItemList[$parentId];
		foreach ($itemList as $itemIdCheck => $itemData) {
			$fBottom = false;
			if ($itemIdCheck==$itemId)
				$fBottom=true;
		}
		return $fBottom;
	}

	public function hasChildren ($itemId) {
		return isset ($this->parentItemList[$itemId]) && 
			count ($this->parentItemList[$itemId])>0;
	}
	
	public function getChildrenCount ($itemId) {
		if (isset ($this->parentItemList[$itemId])) {
			return count ($this->parentItemList[$itemId]);
		}
	}

	public function insertItem ($itemData, $fOpenParent=true, $fDB=true) {
		if (!isset ($itemData["parent_id"]))
			$itemData["parent_id"] = 0;

		$parentId = $itemData["parent_id"];
		
		if (!isset ($itemData["sort_pos"]))
			$itemData["sort_pos"] = $this->getMaxSortPos ($itemData["parent_id"])+1;		
		if ($this->treeDB)
			$this->openIfNoChildren ($parentId);

		if ($fDB && $this->treeDB) {			
			$itemId = $this->treeDB->insertItem ($itemData);
			$this->idMax=$itemId;
		}
		else 
		if (!isset ($itemData["item_id"])) {		
			$itemId = ++$this->idMax;
		}
		else {
			$itemId = $itemData["item_id"];
			if ($this->idMax<$itemId)
				$this->idMax=$itemId;
		}
		$itemData["item_id"]=$itemId;
		$this->itemList[$itemId]=$itemData;
		$this->parentItemList[$parentId][$itemId]=&$this->itemList[$itemId];
		$this->updateItemAttribute ($parentId, "is_open", $fOpenParent ? 1 : 0);
		return $itemId;
	}

	public function updateItem ($itemData) {
		if (!isset ($itemData["item_id"]))
			return false;
		if (!isset ($itemData["parent_id"]))
			$itemData["parent_id"] = 0;

		if ($this->treeDB) {
			$this->treeDB->updateItem ($itemData);
		}
			
		$this->itemList[$itemData["item_id"]]=$itemData;
		return true;
	}
	
	public function updateItemAttribute ($itemId, $attr, $value, $fDB=false) {

		if ($fDB && $this->treeDB) {
			$this->treeDB->updateItemAttribute ($itemId, $attr, $value);
		}
		
		if (!isset ($this->itemList[$itemId]))
			return false;

		$this->itemList[$itemId][$attr]=$value;
		return true;
	}
	
	public function removeItemAttribute ($itemId, $attr) {
		if (!isset ($this->itemList[$itemId]))
			return false;
		unset ($this->itemList[$itemId][$attr]);
		return true;
	}
	
	private function openIfNoChildren ($parentId) {
		// Check if children loaded			
		$parentItemData = $this->getItem ($parentId);
		if ($parentItemData && 
				isset ($parentItemData["show_open"]) &&
				$parentItemData["show_open"]==1 && 
				$this->getChildrenCount ($parentId)==0) {
			$this->openItem ($parentId);
		}
	}
	
	public function moveItem ($itemId, $newParentId) {
		if (!isset ($this->itemList[$itemId]))
			return false;
			
		if ($this->treeDB) {
			$this->openIfNoChildren ($newParentId);
		}
			
		$itemData = $this->itemList[$itemId];
		$newSortPos = $this->getMaxSortPos ($newParentId)+1;
		$oldParentId = $itemData["parent_id"];
		$itemData["parent_id"] = $newParentId;
		$itemData["sort_pos"] = $newSortPos;			
		
		// Remove old parent
		if ($this->getChildrenCount ($oldParentId)==1)		
			$this->updateItemAttribute ($oldParentId, "show_open", 0, true);
		
		unset ($this->parentItemList[$oldParentId][$itemId]);		
		
		// Set item itself
		$this->updateItem ($itemData);

		// Set new parent		
		$this->updateItemAttribute ($newParentId, "show_open", 1, true);
		$this->updateItemAttribute ($newParentId, "is_open", 1, false);
		$this->parentItemList[$newParentId][$itemId]=&$this->itemList[$itemId];

		return true;
	}
	
	public function removeItem ($itemId, $fDB=true) {

		if ($fDB && $this->treeDB) {
			$this->treeDB->removeItem ($itemId);
		}	
		
		if (!isset ($this->itemList[$itemId]))
			return false;

		$this->removeChildren ($itemId, $fDB);
		$parentId = $this->itemList[$itemId]["parent_id"];
		unset ($this->itemList[$itemId]);
		unset ($this->parentItemList[$parentId][$itemId]);
		if ($this->selectedItemId==$itemId)
			$this->selectedItemId=0;
		if (!$this->hasChildren ($parentId)) {
			$this->updateItemAttribute ($parentId, "show_open", 0);
		}

		return true;
	}

	public function removeChildren ($parentId, $fDB=true) {
		if (!isset ($this->parentItemList[$parentId]))
			return false;
		foreach ($this->parentItemList[$parentId] as $itemId => $itemData) {
			$this->removeItem ($itemId, $fDB);
		}
		return true;
	}
	
	public function iterateItems ($parentId, $class, $fWithParent=false) {

		$cnt=0;
		if (!isset ($this->parentItemList[$parentId])) {
			$this->openItem ($parentId);
			if (!isset ($this->parentItemList[$parentId])) {
				if ($fWithParent) {
					$class->onIterateItem($this, $parentId, $this->getItem ($parentId));
					return 1;
				}
				return 0;				
			}
		}

		foreach ($this->parentItemList[$parentId] as $itemId => $itemData) {
		
			$class->onIterateItem($this, $itemId, $itemData);
			$cnt++;
			$cnt+=$this->iterateItems ($itemId, $class);
		}

		if ($fWithParent) {
			$class->onIterateItem($this, $parentId, $this->getItem ($parentId));
			$cnt++;
		}
		return $cnt;
	}
	
	public function getFirstChildItem ($parentId) {
		if (!isset ($this->parentItemList[$parentId]))
			return NULL;
		foreach ($this->parentItemList[$parentId] as $itemId => $itemData) {
			return $itemData;
		}		
	}
	
	public function getItems ($parentId) {
		if (isset ($this->parentItemList[$parentId]))
			return $this->parentItemList[$parentId];
		return NULL;
	}

	public function getItem ($itemId) {
		if (isset ($this->itemList[$itemId]))
			return $this->itemList[$itemId];
		return NULL;
	}
	
	public function isItem ($itemId) {
		return isset ($this->itemList[$itemId]);
	}
	
	public function getAllItems () {
		return  $this->itemList;
	}
	
	public function getItemCount () {
		return count ($this->itemList);
	}
	
	public function getItemPos ($itemId) {
		if (!isset ($this->itemList[$itemId]))
			return false;
		$parentId = $this->itemList[$itemId]["parent_id"];		
		return array_search($itemId,array_keys($this->parentItemList[$parentId]));
	}
	
	public function getNextItem ($itemId) {
		return $this->getRelativeItem ($itemId, 1);
	}
	
	public function getPrevItem ($itemId) {
		return $this->getRelativeItem ($itemId, -1);
	}	
	
	public function getRelativeItem ($itemId, $pos) {
		if (!isset ($this->itemList[$itemId]))
			return NULL;
		$parentId = $this->itemList[$itemId]["parent_id"];
		$nextItemId = $this->keyRelative ($this->parentItemList[$parentId], $itemId, $pos);
		if ($nextItemId!==false)
			return $this->itemList[$nextItemId];
		return NULL;
	}

	public function sort ($itemId, $direction) {
		if (!isset ($this->itemList[$itemId]))
			return false;
		$this->updateItemAttribute ($itemId, "is_marked_once", 1);
		$itemData = $this->itemList[$itemId];				
	
		$parentId = $itemData["parent_id"];

		$itemData2 = $this->getRelativeItem ($itemId, $direction ? 1 : -1);
		if (!$itemData2)
			return false;
			
		$itemId2 = $itemData2["item_id"];
		$sortPos  = $itemData["sort_pos"];
		$sortPos2 = $itemData2["sort_pos"];
		
		$this->updateItemAttribute ($itemId, "sort_pos", $sortPos2, true);
		$this->updateItemAttribute ($itemId2, "sort_pos", $sortPos, true);
	
		uasort ($this->parentItemList[$parentId], array($this, "cmpFunc"));
		
		return array ($itemId => $sortPos2, $itemId2 => $sortPos);
	}
	
	public function selectItem ($itemId) {
		$this->selectedItemId = $itemId;
	}
	
	public function getSelectedItemId () {
		return $this->selectedItemId;
	}
	
	public function getSelectedItem () {
		return $this->getItem ($this->selectedItemId);
	}
	
	public function updateSelectedItem ($itemData) {
		if (!$this->selectedItemId)
			return false;
		$this->itemList[$this->selectedItemId]=$itemData;
		if ($this->treeDB) {
			$this->treeDB->updateItem ($itemData);
		}
		return true;
	}
	
	public function getDB () {
		return $this->treeDB;
	}
	
	private function keyRelative ($array, $current_key, $offset = 1) {
    $keys = array_keys($array);
    $current_key_index = array_search($current_key, $keys);
    if(isset($keys[$current_key_index + $offset])) {
        return $keys[$current_key_index + $offset];
    }
    return false;
	}
	
	private function cmpFunc ($value1, $value2)  {
		return $value1["sort_pos"]>$value2["sort_pos"];
	}	
};

?>