<?php


class UniversalList extends WUIList implements UniversalConnector {
	
	private $props = NULL;
	private $listDef = NULL;
	private $sort_pos_field = "";
		
	public function __construct (&$props) {
		
		$this->props = &$props;	
		$this->parseListDef ();
		parent::__construct($this->props["name"]."_list", self::RESET_NEVER);
	}
	
	public function __destruct () {
	
	}
	
	private function getSortColIdxByName ($column) {
		$idx = 1;
		foreach ($this->props["list_columns"] as $column2 => $columnProps) {
			if ($column2==$column)
				return $idx;
			$idx++;
		}
		return 1;
	}
	
	
	private function parseListDef () {
		$this->listDef["sort_col_id"] = $this->getSortColIdxByName ($this->props["sort_column"]);	
		$this->listDef["sort_direction"] = $this->props["sort_direction"];
		$this->listDef["edit_params"] = array ("view" => WSystem::getArrayValue ($this->props, "list_view_cmd_edit", "edit"));
		$this->listDef["columns"] = $this->props["list_columns"];
		// Optional defs
		if (isset ($this->props["list_view"])) {
			$this->listDef["listview"] = $this->props["list_view"];		
			if (strpos ($this->listDef["listview"], "sort")!==false)
				$this->sort_pos_field = "sort_pos";
		}
		
		if (isset ($this->props["list_width"]))
			$this->listDef["width"] = $this->props["list_width"];		
	}

	public function getDefinition () {
		$searchData = $this->getSearchData ();		
		foreach ($searchData as $field => $value) {
			if (isset ($this->props["search_fields"][$field]["list_view"]) && !empty ($value)) {				

				$this->listDef["listview"] = $this->props["search_fields"][$field]["list_view"];
				$this->sort_pos_field = $field;		

			}
		}
		return $this->listDef;
	}
	
	private function getTableColumn ($table, $field, $fSortPos=false, $fNoAs=false) {


		if ((strpos ($field, "-"))!==false) {
			list ($table2, $field2)=explode ("-", $field);
			$column = DB_TABLE_PREFIX.$table2.".".$field2;

		}		
		else
		if ((strpos ($field, "."))!==false) {
			list ($table2, $field2)=explode (".", $field);
			$column = DB_TABLE_PREFIX.$table2.".".$field2;
			if (!$fNoAs)
				$column.=" as ".$table2."_".$field2;
		}
		else		
		if (strpos ($field, ":")!==false) {				
			
			list ($table2, $field2) = explode (":", $field);
			if ($fSortPos)
				$field2="sort_pos";
			
			$relation = $this->props["relations"][$table2]["relation"];
			if ($relation=="n:m")				
				$column=DB_TABLE_PREFIX.$table."2".$table2.".".$field2;
			else
				$column=DB_TABLE_PREFIX.$table2.".".$field2;
		}
		else { 
			$column=DB_TABLE_PREFIX.$table.".".($fSortPos ? "sort_pos" : $field);
		}
		return $column;
	}
	
	public function loadDataList () {

		global $wdb;
		$dataList = array ();
		$idx=0;
		
		// Main table
		$table = $this->props["name"];
		
		// Fields
		$sqlFields = DB_TABLE_PREFIX.$table.".".$this->props["primary_column"];
		if (!empty ($this->sort_pos_field)) {
			$sqlFields.=",".$this->getTableColumn ($table, $this->sort_pos_field, true);
		}
		
		foreach ($this->listDef["columns"] as $field => $fieldProps) {
			if (strpos ($field, "~")!==false)
				continue;
			
			if (strpos ($field, "+")!==false) 
				$sep ="+";
			if (strpos ($field, "|")!==false) 
				$sep ="|";
			if (!empty ($sep)) {
				$list = explode ($sep, $field);
				foreach ($list as $field2)
					$sqlFields.=",".$this->getTableColumn ($table, $field2);
			}
			else {
				$sqlFields.=",".$this->getTableColumn ($table, $field);
			}
		}

		$sql = "select ".$sqlFields." from ";
		
		// Tables		
		$sql.=DB_TABLE_PREFIX.$this->props["name"].$sqlJoin;

		// All relation tables
		$groupBy = "";

		if (isset($this->props["relations"])) {
			foreach ($this->props["relations"] as $table2 => $table2Props) {
				$relation = $this->props["relations"][$table2]["relation"];
				$foreign_column = $this->props["relations"][$table2]["primary_column"];
				if ($relation=="n:m")	{
					$sql.=" left join ".DB_TABLE_PREFIX.$table."2".$table2." on (".DB_TABLE_PREFIX.$table."2".$table2.".".$this->props["primary_column"]."=".DB_TABLE_PREFIX.$table.".".$this->props["primary_column"].")";
					$groupBy=DB_TABLE_PREFIX.$table.".".$this->props["primary_column"];
				}
				else
					$sql.=" left join ".DB_TABLE_PREFIX.$table2." on (".DB_TABLE_PREFIX.$table2.".".$foreign_column."=".DB_TABLE_PREFIX.$table.".".$foreign_column.")";
			}
		}


		// Where query
		$searchData = $this->getSearchData ();

		$sqlWhere = WSystem::getArrayValue ($this->props, "search_sql_where");
		if (!empty ($sqlWhere))
			$sqlWhere = " where ".$sqlWhere;

		if (isset ($this->props["search_query_handler"])) {
			$this->props["search_query_handler"]($sqlWhere, $this->props["search_fields"], $searchData);
		}
		else
		{

			foreach ($this->props["search_fields"] as $field => $seachOptions) {
				if (isset ($searchData[$field]) && !empty ($searchData[$field])) {
					
					$column = $this->getTableColumn ($table, $field);				
					
					$search = $searchData[$field];
					if (isset ($seachOptions["bool_search"]) && $seachOptions["bool_search"]) { // Checkbox!
						if ($search==0)
							continue;
						if ($search==1)
							$search=0;
						else
							$search=1;
					}
					$otherColumns = "";
					if (isset ($seachOptions["other_columns"]))
						$otherColumns = $seachOptions["other_columns"];
					WDB::addWhereClause ($sqlWhere, $column, $search, (isset ($seachOptions["is_like"]) ? $seachOptions["is_like"] : 0), $otherColumns);
					
				}
			}
		}		

		$sql.=$sqlWhere;

		// Group by
		if (!empty ($groupBy)) {
			$sql.=" group by ".$groupBy;
		}

		// Order by
		if (empty ($this->sort_pos_field)) {
		
			$sql.=" order by ".$this->getTableColumn ($table, $this->props["sort_column"], false, true)." ".($this->props["sort_direction"] ? "ASC" : "DESC");

		}
		else
			$sql.=" order by ".$this->getTableColumn ($table, $this->sort_pos_field, true, true);

		// Limit
		$sql.=" limit ".$this->props["search_limit"];
		
		if ($wdb->query ($sql)) {
			$data = array ();
			while ($wdb->fetch ($data)) {

				if ($this->rowDataFunc) {
					$tmp = call_user_func ($this->rowDataFunc, $data);
					if ($tmp!=NULL) {				
						$data = $tmp;
					}
				}

				$dataList[$idx++]=$data;
			}
		}
		return $dataList;
	}
	
	public function selectItem ($nav_idx) {
		$data = $this->getData ($nav_idx);		
		if (!isset ($data["is_selected"]) || !$data["is_selected"]) {
			$data["is_selected"]=1;
		}
		else {
			$data["is_selected"]=0;
		}
		$this->setData ($nav_idx, $data);
	}
	
	public function swapSortPos ($nav_idx, $nav_idx2) {
		global $wdb;
		$searchData = $this->getSearchData ();

		$data = $this->getData ($nav_idx);
		$data2 = $this->getData ($nav_idx2);
		
		$table = $this->props["name"];
		
		if (strpos ($this->sort_pos_field, ":")!==false) {				
			list ($table2, $field2) = explode (":", $this->sort_pos_field);			
			$relation = $this->props["relations"][$table2]["relation"];
			$foreign_column = $this->props["relations"][$table2]["primary_column"];
			if ($relation=="n:m")	{
				$column=DB_TABLE_PREFIX.$table."2".$table2.".".$field2;
			
				$sql = "update ".DB_TABLE_PREFIX.$table."2".$table2." set sort_pos=".$data["sort_pos"]." where ".
					$foreign_column."=".$searchData[$table2.":".$foreign_column]." and ".$this->props["primary_column"]."=".$data2[$this->props["primary_column"]];
				$wdb->query ($sql);
				
				$sql = "update ".DB_TABLE_PREFIX.$table."2".$table2." set sort_pos=".$data2["sort_pos"]." where ".
					$foreign_column."=".$searchData[$table2.":".$foreign_column]." and ".$this->props["primary_column"]."=".$data[$this->props["primary_column"]];
				$wdb->query ($sql);
			
			}
			else {
				// No sorting possible!
				return;
			}
		}
		else {

			$sql = "update ".DB_TABLE_PREFIX.$table." set sort_pos=".$data["sort_pos"]." where ".
				$this->props["primary_column"]."=".$data2[$this->props["primary_column"]];

			$wdb->query ($sql);
			
			$sql = "update ".DB_TABLE_PREFIX.$table." set sort_pos=".$data2["sort_pos"]." where ".
				$this->props["primary_column"]."=".$data[$this->props["primary_column"]];
			$wdb->query ($sql);
		}				
				
				
		$tmp = $data["sort_pos"];
		$data["sort_pos"]=$data2["sort_pos"];
		$data2["sort_pos"] = $tmp;
		$this->setData ($nav_idx, $data2);
		$this->setData ($nav_idx2, $data);
	}
	
	
	// Connector functions
	public function getPrimaryId () {
		$data = parent::getSelectedData ();
		if ($data)
			return $data[$this->props["primary_column"]];
		return 0;
	}
	
	public function select () {
		return parent::getSelectedData ();;
	}

	public function update (&$values) {
		$data = parent::getSelectedData ();
		if ($data) {
			// Replace only existing keys
			foreach ($data as $field => $value) {
				if (isset ($values[$field])) {
					$data[$field] = $values[$field];
				}
			}
			parent::setSelectedData ($data);
		}
	}

	
	public function insert (&$values) {
		parent::reset ();	
		parent::setHighlightNew ($this->props["primary_column"], $values[$this->props["primary_column"]]);
		
	}
		
	public function reset ($searchData=array()) {
		parent::reset ($searchData);
	}
	
	public function delete () {
		$data = parent::getSelectedData ();
		$data["is_deleted"]=true;
		parent::setSelectedData ($data);			
	}
	
}
?>