<?php



class UniversalLogic {
	public $props = array ();
	public $inputDef = array ();
	public $fileDef = array ();
	public $elementDef = array ();
	public $errText = "";
	public $fTreeView = false;
	protected $primary_id = 0;

	public function __construct (&$props) {
		// Basic checks
		if (!isset ($props["primary_column"])) {
			$props["primary_column"]=$props["name"]."_id";
		}
		$this->props = &$props;
		$this->fTreeView = isset ($this->props["treeview_map"]);
		$this->fSortable = $this->fTreeView;
		$this->parseInputDef ();
	}


	private function parseInputDef () {
		$this->inputDef = array ();
		$this->fileDef = array ();
		

		
		$this->inputDef[$this->props["primary_column"]] = "i;*";
		if ($this->fTreeView)
			$this->inputDef["parent_".$this->props["primary_column"]] = "i;*";
				
		foreach ($this->props["input_form"] as $tabProps) {
			if (isset ($tabProps["dynamic_content"]))
				continue;
			foreach ($tabProps["input_fields"] as $field => $fieldProps) {
				$field2 = trim ($field, '~');
				if ($field2!=$field) {
					// Only for showing
					continue;
				}
				if ($fieldProps["type"]=="element") {
					$this->elementDef[$field]=$fieldProps;
					continue;
				}
				if (strpos ($field, ":")!==false) {
					// Relatated column
					continue;
				}
				

				$this->inputDef[$field] = $fieldProps["input"];		
				if ($fieldProps["type"]=="upload") {
					$this->fileDef[$field]=$fieldProps;
				}

			}
		}
		// default fields
		$this->inputDef["sort_pos"] = "i;$";	
		$this->inputDef["changed"] = "t;$";
		$this->inputDef["changer_id"] = "i;$";		
		$this->inputDef["created"] = "t;$";
		$this->inputDef["creator_id"] = "i;$";	
		
		
	}
	
	public function saveFiles ($primary_id, &$values) {
		global $wdb;
		foreach ($this->fileDef as $field => $fieldProps) {
			
						
			if (!isset ($values[$field."_tmp"]) || empty ($values[$field."_tmp"]))
				continue;

				
			$pathinfo = pathinfo ($values[$field."_orig"]);
			
			// Create basename ...
			// $name = WUtils::createCompatibleFilename ($values["name"]."_".$primary_id);
			// $name = $primary_id;
			$name = $pathinfo["filename"];

			if (!$this->onLogic ("save_file0", $primary_id, $values, $name, $field))
				return false;
			
			$values[$field] = WUtils::moveUploadFile ($values[$field."_tmp"], $values[$field], $fieldProps["upload_path"], $name);			
			$this->onLogic ("save_file1", $primary_id, $values, $name, $field);

			// Update new filename and file size
			$sqlFileSize = ",".$field."_size='".WDB::escape ($values[$field."_size"])."'";

			
			$sql = "update ".DB_TABLE_PREFIX.$this->props["name"]." set ".$field."='".WDB::escape ($values[$field])."'".$sqlFileSize." where ".$this->props["primary_column"]."='".WDB::escape ($primary_id)."'";				
			$wdb->query ($sql);

			if (!$this->onLogic ("save_file2", $primary_id, $values, $name, $field))
				return false;
		}
	}
	
		
	public function deleteFiles ($primary_id, &$values, $onlyField=NULL) {
		foreach ($this->fileDef as $field => $fieldProps) {
		
			if ($onlyField && $field!=$onlyField)
				continue;
				
			if (isset ($values[$field])) {
				if (!$this->onLogic ("delete_file0", $primary_id, $values, $field))
					return false;
				
				WUtils::removeFile (ROOT_PATH."/".$fieldProps["upload_path"]."/".$values[$field]);
			
				$tmp = "";
				if (!$this->onLogic ("delete_file2", $primary_id, $values, $tmp, $field))
					return false;
			}
		}		
	}

	private function checkDuplicates ($primary_id, &$values) {
		global $wdb;
		if (!isset ($this->props["dup_checks"]))
			return false;
	
		foreach ($this->props["dup_checks"] as $field => $dupProps) {
			$field2 = $field;
			if (isset ($dupProps["value_field"]))
				$field2 = $dupProps["value_field"];
			
			$andClause="";
			if (strpos ($field, ",")!==false) {
				$tmpList = explode (",", $field);
				$i = 0;
				foreach ($tmpList as $tmp) {
					if ($i==0) {
						$field=$field2=$tmp;
					}
					else {
						$andClause.=' and '.$tmp.'="'.WDB::escape ($values[$tmp]).'" ';
					}
					$i++;
				}
			}

			if ($wdb->checkDuplicate (DB_TABLE_PREFIX.$this->props["name"], $field, $values[$field2], $this->props["primary_column"], $primary_id, "", $andClause)) {				
				$this->errText = WUtils::templateText ($dupProps["err_text"], $values);
				return true;
			}
		}
		return false;
	}

	public function insert (&$values)
	{
		global $wdb, $session;
		// Check duplicates
		if ($this->checkDuplicates (0, $values)) {
			return false;
		}
		
		// Timestamps
		$values["created"]=WDB::getDate ();
		$values["creator_id"]=$session->getUserId ();		
		
		// Do some specific treeview stuff
		if ($this->fTreeView) {
			// Set sort_pos
			$sql = "select max(sort_pos) as max_sort_pos from ".DB_TABLE_PREFIX.$this->props["name"]." where parent_".$this->props["primary_column"]."=".$values["parent_".$this->props["primary_column"]];
			$values["sort_pos"] = $wdb->getId ($sql, "max_sort_pos")+1;
		}
		
		$ret = $this->onLogic ("insert0", $primary_id, $values);		
		if ($ret==false)
			return false;
		
		// Insert into db
		if (strcmp ($ret, "NO_DB_ACTION")!=0)
			$primary_id = $wdb->insertFields ($this->inputDef, DB_TABLE_PREFIX.$this->props["name"], $values);		
		else
			$primary_id = $this->primary_id;
			
		// Do some specific treeview stuff
		if ($this->fTreeView) {
			// Set parent item has_children
			$sql = "update ".DB_TABLE_PREFIX.$this->props["name"]." set has_children=1 where ".$this->props["primary_column"]."=".$values["parent_".$this->props["primary_column"]];
			$wdb->query ($sql);
		}

		
		// Ok, we have a primary key
		$values[$this->props["primary_column"]]=$primary_id;
		
		
		if (!$this->onLogic ("insert1", $primary_id, $values))
			return false;
		
		// Insert specific elements (relations)
		foreach ($this->elementDef as $field => $fieldProps) {
			$element = &$fieldProps["element"];
			
			$props =  $this->props;
			$props["values"] =  $values;
			$element->init ($field, $primary_id, $props, $fieldProps);
			if (!isset ($values[$field])) {
				$inputValues = $element->getInput ();				
				$element->insert ($inputValues);
			}
			else
				$element->insert ($values[$field]);
		}
				
		// Save files
		$this->saveFiles ($primary_id, $values);
		
		if (!$this->onLogic ("insert2", $primary_id, $values))
			return false;
		
		return $primary_id;
	}
	
	public function update ($primary_id, &$values, $fRelations=true) {
		global $wdb, $session;
		// Check duplicates
		if ($this->checkDuplicates ($primary_id, $values)) {
			return false;
		}
		
		// Timestamps
		$values["changed"]=WDB::getDate ();
		$values["changer_id"]=$session->getUserId ();	
		
		if (!$this->onLogic ("update0", $primary_id, $values))
			return false;
		
		// Save files
		$this->saveFiles ($primary_id, $values);

		// Update db


		$wdb->updateFields ($this->inputDef, DB_TABLE_PREFIX.$this->props["name"], $values, $this->props["primary_column"], $primary_id);
		if (!$this->onLogic ("update1", $primary_id, $values))
			return false;

		
		// Update specific elements (relations)
		if ($fRelations) {
			foreach ($this->elementDef as $field => $fieldProps) {
				$element = &$fieldProps["element"];
				$element->init ($field, $primary_id, $this->props, $fieldProps);
				if (!isset ($values[$field]))	{
					$inputValues = $element->getInput ();
					$element->update ($inputValues);
				}
				else
					$element->update ($values[$field]);
			}
		}		
		if (!$this->onLogic ("update2", $primary_id, $values))
			return false;
			
		return true;	
	}
	
	public function delete ($primary_id, &$values=NULL) {
		global $wdb;
	
		// Delete all childs recursive in treeviews
		if ($this->fTreeView) {
			$childList = $wdb->queryTable (DB_TABLE_PREFIX.$this->props["name"], $this->props["primary_column"], "where parent_".$this->props["primary_column"]."=".$primary_id, $this->props["primary_column"]);
			foreach ($childList as $child_id => $childData) {
				$this->delete ($child_id);
			}
		}
		
		if (! ($values = $this->select ($primary_id))) {
			return false;
		}
		
		$ret = $this->onLogic ("delete0", $primary_id, $values);
		if ($ret==false)
			return false;
			
		// Delete from DB
		if (strcmp ($ret, "NO_DB_ACTION")!=0)
			$wdb->delete (DB_TABLE_PREFIX.$this->props["name"], $this->props["primary_column"], $primary_id);		

		// Do some specific treeview stuff
		if ($this->fTreeView) {
			// Set parent item has_children
			$sql = "select count(*) as cnt from ".DB_TABLE_PREFIX.$this->props["name"]." where parent_".$this->props["primary_column"]."=".$values["parent_".$this->props["primary_column"]];
			if (!$wdb->getId ($sql, "cnt")) {
				$sql = "update ".DB_TABLE_PREFIX.$this->props["name"]." set has_children=0 where ".$this->props["primary_column"]."=".$values["parent_".$this->props["primary_column"]];
				$wdb->query ($sql);	
			}
		}
		if (!$this->onLogic ("delete1", $primary_id, $values))
			return false;
			
		// Delete specific elements (relations)
		foreach ($this->elementDef as $field => $fieldProps) {
			$element = &$fieldProps["element"];
			$element->init ($field, $primary_id, $this->props, $fieldProps);
			$element->delete ();
		}

		// Delete files
		$this->deleteFiles ($primary_id, $values);
		if (!$this->onLogic ("delete2", $primary_id, $values))
			return false;
			
		return true;
	}
	
	public function select ($primary_id, $fRelations=false) {
		global $wdb;
		
		$data = array ();
		if (!$this->onLogic ("select0", $primary_id, $data, $fRelations))
			return NULL;

		if (!$wdb->queryRow (DB_TABLE_PREFIX.$this->props["name"], "where ".$this->props["primary_column"]."='".WDB::escape($primary_id)."'", $data)) {			
			return NULL; // throw new Exception ("Database entry not found.");
		}

		if (!$this->onLogic ("select1", $primary_id, $data, $fRelations))
			return NULL;
		if ($fRelations) {
			// Delete specific elements (relations)
			foreach ($this->elementDef as $field => $fieldProps) {
				$element = &$fieldProps["element"];
				$element->init ($field, $primary_id, $this->props, $fieldProps);
				$data[$field]=$element->select ();
			}
		}
		if (!$this->onLogic ("select2", $primary_id, $data, $fRelations))
			return NULL;
		return $data;
	}		
	
	public function onLogic ($cmd, $primary_id, &$values, &$value2=NULL, $field="") {
		// Can be overwritten for own logic
		return true;
	
	}
	
}

?>