<?php

class UniversalTreeview extends WTreeview implements UniversalConnector
{
	private $props = NULL;
	private $logic = NULL;
	private $profile_id = 0;
	private $fMove = false;
			
	public function __construct (&$props, &$logic, $fMove=false, $profile_id=0) {
	
		$this->props = &$props;
		$this->logic = &$logic;
		$this->profile_id = $profile_id;
		$this->fMove = $fMove;
		if ($fMove) {
			$id.="_move";
			parent::__construct ($props["name"]."_treeview", $this->props["texts"]["treeview_root"], NULL, NULL);
		} 
		else {
			parent::__construct ($props["name"]."_treeview_move", $this->props["texts"]["treeview_root"], $this->props["context_menu2"], $this->props["context_menu"]);
		}
		
		$this->setIconMap (array (
			0 => array ("images/tree/folder.gif", 18, 18),
			1 => array ("images/tree/folder_gray.gif", 18, 18)
		));
	}
	
	public function __destruct () {
	
	}
	
	public function onItemData (&$itemData) {
		$itemData["icon_id"]=0;
		if (isset ($itemData["is_visible"]) && !$itemData["is_visible"])
			$itemData["icon_id"]=1;
			
		$itemData["item_title"]="ID ".$itemData["item_id"];
	}
	
	public function loadTree () {
		global $wdb;

		if (!$this->fMove)
			$this->profile_id = WSystem::getParamFromSession ($this->id."profile_id", $this->profile_id );

		$defaultMap =  array (	
			"item_id"     => $this->props["primary_column"],
			"parent_id"   => "parent_".$this->props["primary_column"],														
			"sort_pos"    => "sort_pos",
			"profile_id"    => "profile_id",
			"show_open"   => "has_children",
			"is_visible"  => "is_visible");
			
		$this->tree = new WTree (new TreeDB ($wdb, DB_TABLE_PREFIX.$this->props["name"], array_merge ($defaultMap,$this->props["treeview_map"]), "profile_id='".$this->profile_id."' and is_deleted=0"));


		if (WSystem::getParam ("view")=="init" || WSystem::getParam ("cmd")=="change_profile") {
			unset ($_SESSION[$this->id."treeData"]);
		}
		
		if (isset ($_SESSION[$this->id."treeData"])) {
			$this->tree->unserialize ($_SESSION[$this->id."treeData"]);
		}
		else {
			$this->tree->initFromDB ();
		}
	}

	public function getProfileId () {
		return $this->profile_id;
	}
	
	public function saveTree () {
		// Save tree into session
		$_SESSION[$this->id."treeData"]=$this->tree->serialize ();		
	}
	
	public function createCommandBar () {
		if (!$this->contextMenu)
			return "";
		return "";
	}

	public function resetTree () {
		unset ($_SESSION[$this->id."treeData"]);
		$this->tree->initFromDB();
		$this->saveTree ();
	}
	
	public function onTreeEvent ($cmd) {
		global $detailForm, $treeView, $session;
		switch ($cmd) {
			case "open":
				if (!$this->tree->isItem ($this->getItemId ()))
					$treeView->showAndClosePopup ("popup_treeview_move");
				
				$this->tree->openItem ($this->getItemId ());
				break;
	
			case "close":
				if (!$this->tree->isItem ($this->getItemId ()))
					$treeView->showAndClosePopup ("popup_treeview_move");
					
				$this->tree->closeItem ($this->getItemId ());				
				break;
				
			case "select":
				WUI::commandAsync (array ("freePageInstances" => 1));	
				if ($this->contextMenu==NULL) {
					$itemId    = $this->getItemId ();
					$scrItemId = $this->tree->getIgnoreItemId ();	
					$itemData  = $this->tree->getItem ($itemId);
					/*
					if (!$this->tree->isItem ($scrItemId)) {
						WUI::closePopup ("popup_treeview_move");
					}
					*/
					if ($itemId!=0 && !$itemData) {
						WUI::closePopup ("popup_treeview_move");
					}
					if (!WUI::isConfirm ()) {
					
						$question = $session->getLangVar ("MOVE_TO_QUESTION", "Wollen Sie auf '{name}' verschieben?");
						if ($itemId==0)
							$itemData["name"]=$treeView->getRootName ();

										
						WUI::setConfirm (WUtils::templateText ($question, $itemData));						
					}
					else {
						// Move selected item to
						$treeView->getTree ()->moveItem ($scrItemId, $itemId);
						
						$detailForm->show (true);
						$treeView->showAndClosePopup ("popup_treeview_move");
					
					}
				}
				else
					$detailForm->show ();
				break;
				
		
		}
		return true;
	}


	public function onMenuEvent ($cmd, $param) {
		global $detailForm, $treeViewMove, $session;
		$itemId = (int) $param;
		if (!empty ($cmd) && $itemId && !$this->tree->isItem ($itemId)) {
			$this->onTreeEvent ("select");
			return true;
		}
		switch ($cmd) 
		{
			case 'REMOVE':
				
				$selectedItemId = $this->tree->getSelectedItemId ();
				$itemData = $this->tree->getItem ($itemId);
				if (!WUI::isConfirm ()) {
					$deleteQuestion = WUtils::templateText (WSystem::getArrayValue ($this->props, "texts",  WLanguage::DELETE_QUESTION, "delete_question"), $itemData);
					WUI::setConfirm ($deleteQuestion);
				}
				else {
					WUI::closePopupAsync ("popup_".$this->props["name"]."_*");
					if ($this->logic->delete ($itemId)) {						

						$this->tree->removeItem ($itemId, false);						

						$successText = WUtils::templateText (WSystem::getArrayValue ($this->props, "texts", DELETE_ENTRY_DELETED, "delete_success"), $itemData);
						WUI::setMessage ($successText);
						
						$detailForm->show ();
					}
				}
				return true;
				
				break;
				
			case 'EDIT':
				/*
				$itemData = $this->tree->getItem ($itemId);
				$pageForm = new PageForm ("edit", $this);
				WUI::openPopup ("popup_".$this->props["name"]."_dlg", "'".$itemData["name"]."' bearbeiten", $pageForm->__toString (), array ("is_moveable"=>1, 'addClass' => array ('item-'.$this->id.'_'.$itemId, 'arrow')));
				return true;
				*/
				break;				
				
			case 'MOVE_DOWN':
				$itemData = $this->tree->sort ($itemId, true);
				return true;
				break;
				
			case 'MOVE_UP':

				$itemData = $this->tree->sort ($itemId, false);
				return true;
				break;
		
			case 'NEW':
				/*
				$pageForm = new PageForm ("new", $this);
				WUI::openPopup ("popup_".$this->props["name"]."_dlg", "Neu anlegen", $pageForm->__toString (), array ('is_moveable'=>1, 'addClass' => array ('item-'.$this->id.'_'.$itemId, 'arrow')));
				return true;
				*/
				break;

			case 'MOVE':
				$itemData = $this->tree->getItem ($itemId);				
				$treeViewMove->resetTree ();
				$treeViewMove->getTree ()->setIgnoreItemId ($itemId);
				$treeViewMove->renderElement ();
				
				$title  = WUtils::templateText ($session->getLangVar ("MOVE_TO", "'{name}' verschieben ..."), $itemData);
				
				WUI::openPopup ("popup_treeview_move", $title, $treeViewMove->__toString (), array ('is_moveable'=>1, 'addClass' => array ('item-'.$this->id.'_'.$itemId, 'arrow')));
				return true;
				break;
		}
		return false;
	}
	
	
	// Connector functions	
	public function getPrimaryId () {
		return $this->tree->getSelectedItemId ();
	}
	
	public function select () {
		return $this->tree->getSelectedItem ();
	}
	
	public function update (&$values) {
		// Map values back to treeview conform values
		$values2 = $this->tree->getDB ()->remapValues ($values);		
		
		// Set new data
		$this->tree->updateSelectedItem (array_merge ($this->tree->getSelectedItem (), $values2));
		
		// Show treeview
		$this->show (true);
	}
	
	public function insert (&$values) {
		// Map values back to treeview conform values
		$values2 = $this->tree->getDB ()->remapValues ($values);		
	
		// Insert (its just in database)
		$this->tree->insertItem ($values2, true, false);
		
		// Select and show treeview
		$this->tree->selectItem ($values2["item_id"]);
		
		$this->show (true);
	}
	
	public function setHighlightNew ($primary_id) {
		
	}
	
	public function delete () {
		// Remove (it was just deleted from db)
		$this->tree->removeItem ($this->getPrimaryId (), false);

		WUI::closePopupAsync ("popup_*");
		$this->show (true);		
	}
	
	public function reset ($searchData=array()) {
	
	
	}
}



?>